#! /usr/bin/env python
# -*- coding:utf8 -*-
#
# elastic.py
#
# This file is part of pymls, a software distributed under the MIT license.
# For any question, please contact one of the authors cited below.
#
# Copyright (c) 2017
# 	Olivier Dazel <olivier.dazel@univ-lemans.fr>
# 	Mathieu Gaborit <gaborit@kth.se>
# 	Peter Göransson <pege@kth.se>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#

from .medium import Medium
from numpy import sqrt


class Elastic(Medium):

    MEDIUM_TYPE = 'elastic'
    MODEL = MEDIUM_TYPE
    EXPECTED_PARAMS = [
        ('E', float),  # Young's modulus
        ('nu', float),  # Poisson ratio
        ('rho', float),  # Density
        ('eta', float),  # loss factor
    ]
    OPT_PARAMS = [
        ('lambda_', complex),
        ('mu', complex),
        ('law', str)
    ]

    def __init__(self, **params):
        self.E = None
        self.rho = None
        self.nu = None
        self.eta = None
        self.lambda_ = None
        self.mu = None
        self.delta_p = None
        self.delta_s = None
        self.law = None

        super().__init__(**params)

    def _compute_missing(self):
        if self.law is None:
            self.law = "structural"
            self.E *= (1+1j*self.eta)
        if self.lambda_ is None:
            self.lambda_ = (self.E*self.nu)/((1+self.nu)*(1-2*self.nu))
        if self.mu is None:
            self.mu = (self.E)/(2*(1+self.nu))

    def update_frequency(self, omega):
        self.omega = omega
        if self.law == "rubber":
            self.E = (self.E.real)+1j*omega*self.eta
            self.lambda_ = (self.E*self.nu)/((1+self.nu)*(1-2*self.nu))
            self.mu = (self.E)/(2*(1+self.nu))
        P_mat = self.lambda_ + 2*self.mu
        self.delta_p = omega*sqrt(self.rho/P_mat)
        self.delta_s = omega*sqrt(self.rho/self.mu)



#! /usr/bin/env python
# -*- coding:utf8 -*-
#
# test_loading.py
#
# This file is part of mediapack, a software distributed under the MIT license.
# For any question, please contact one of the authors cited below.
#
# Copyright (c) 2020
# 	Olivier Dazel <olivier.dazel@univ-lemans.fr>
# 	Mathieu Gaborit <gaborit@kth.se>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#

import pytest
import tempfile
import os

from mediapack.utils import from_yaml
from mediapack import Air, EqFluidJCA, Fluid, PEM, Elastic
from mediapack.medium import Medium


class TestMediaLoading:

    @pytest.fixture
    def yaml_fn(self, tmpdir):
        path =  tmpdir.join('medium.yaml')
        path.ensure()
        return path.strpath

    def test_file_not_found(self, tmpdir):

        with pytest.raises(IOError):
            from_yaml(tmpdir.join('inexistent.yaml').strpath)

    def test_bad_medium_type(self, yaml_fn):
        with open(yaml_fn, 'w') as fh:
            fh.write('medium_type: phony_medium\n')

        with pytest.raises(ValueError):
            from_yaml(yaml_fn)

    def test_missing_parameter(self, yaml_fn):
        params = ['phi', 'sigma', 'alpha', 'Lambda_prime']

        with open(yaml_fn, 'w') as fh:
            fh.write('medium_type: eqf\n')
            for p in params:
                fh.write('{}: 42\n'.format(p))

        with pytest.raises(LookupError):
            from_yaml(yaml_fn)

    def test_ok_loading_eqf_withoutopts(self, yaml_fn):
        params = ['phi', 'sigma', 'alpha', 'Lambda_prime', 'Lambda']

        with open(yaml_fn, 'w') as fh:
            fh.write('medium_type: eqf\n')
            for ii, p in enumerate(params):
                fh.write('{}: {}\n'.format(p, ii))

        medium = from_yaml(yaml_fn)
        for ii, p in enumerate(params):
            assert getattr(medium, p) == ii


    def test_ok_loading_eqf_withopts(self, yaml_fn):
        params = ['phi', 'sigma', 'alpha', 'Lambda_prime', 'Lambda',
                  'rho_1', 'nu', 'E', 'eta']

        with open(yaml_fn, 'w') as fh:
            fh.write('medium_type: eqf\n')
            for ii, p in enumerate(params):
                fh.write('{}: {}\n'.format(p, ii))

        medium = from_yaml(yaml_fn)
        for ii, p in enumerate(params):
            assert getattr(medium, p) == ii

    def test_ok_loading_fluid(self, yaml_fn):
        params = ['rho', 'c']

        with open(yaml_fn, 'w') as fh:
            fh.write('medium_type: fluid\n')
            for ii, p in enumerate(params):
                fh.write('{}: {}\n'.format(p, ii))

        medium = from_yaml(yaml_fn)
        for ii, p in enumerate(params):
            assert getattr(medium, p) == ii

    def test_ok_loading_pem(self, yaml_fn):
        params = ['phi', 'sigma', 'alpha', 'Lambda_prime', 'Lambda', 'rho_1', 'nu', 'E',
                  'eta', 'loss_type']

        with open(yaml_fn, 'w') as fh:
            fh.write('medium_type: pem\n')
            for ii, p in enumerate(params):
                if p!='loss_type':
                    fh.write('{}: {}\n'.format(p, ii))
                else:
                    fh.write('{}: structural\n'.format(p))

        medium = from_yaml(yaml_fn)
        for ii, p in enumerate(params):
            if p!='loss_type':
                assert getattr(medium, p) == ii
            else:
                assert getattr(medium, p) == 'structural'

    def test_ok_loading_elastic(self, yaml_fn):
        params = ['E', 'nu', 'rho', 'eta']

        with open(yaml_fn, 'w') as fh:
            fh.write('medium_type: elastic\n')
            for ii, p in enumerate(params):
                fh.write('{}: {}\n'.format(p, ii))

        medium = from_yaml(yaml_fn)
        for ii, p in enumerate(params):
            assert getattr(medium, p) == ii

    def test_failed_forced_loading(self, yaml_fn):
        pem_params = ['phi', 'sigma', 'alpha', 'Lambda_prime', 'Lambda', 'rho_1', 'nu', 'E',
                  'eta', 'loss_type']
        eqf_params = ['phi', 'sigma', 'alpha', 'Lambda_prime', 'Lambda',
                  'rho_1', 'nu', 'E', 'eta']

        with open(yaml_fn, 'w') as fh:
            fh.write('medium_type: pem\n')
            for ii, p in enumerate(pem_params):
                if p!='loss_type':
                    fh.write('{}: {}\n'.format(p, ii))
                else:
                    fh.write('{}: structural\n'.format(p))

        medium = from_yaml(yaml_fn, EqFluidJCA)
        for ii, p in enumerate(eqf_params):
            assert getattr(medium, p) == ii


class TestMediaInstance:

    def test_instanciation_fluid(self):
        params = ['rho', 'c']
        medium = Fluid(**dict(zip(params, range(len(params)))))
        for ii, p in enumerate(params):
            assert getattr(medium, p) == ii

    def test_instanciation_eqf(self):
        params = ['phi', 'sigma', 'alpha', 'Lambda_prime', 'Lambda']
        medium = EqFluidJCA(**dict(zip(params, range(len(params)))))
        for ii, p in enumerate(params):
            assert getattr(medium, p) == ii

    def test_instanciation_elastic(self):
        params = ['E', 'nu', 'rho', 'eta']
        medium = Elastic(**dict(zip(params, range(len(params)))))
        for ii, p in enumerate(params):
            assert getattr(medium, p) == ii

    def test_instanciation_pem(self):
        params = ['phi', 'sigma', 'alpha', 'Lambda_prime', 'Lambda', 'rho_1', 'nu', 'E', 'eta']
        medium = PEM(**dict(zip(params, range(len(params)))), loss_type='structural')
        for ii, p in enumerate(params):
            assert getattr(medium, p) == ii
        assert medium.loss_type == 'structural'



#! /usr/bin/env python
# -*- coding:utf8 -*-
#
# medium.py
#
# This file is part of pymls, a software distributed under the MIT license.
# For any question, please contact one of the authors cited below.
#
# Copyright (c) 2017
# 	Olivier Dazel <olivier.dazel@univ-lemans.fr>
# 	Mathieu Gaborit <gaborit@kth.se>
# 	Peter Göransson <pege@kth.se>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.


class Medium(object):
    """ Base class for medium definition and manipulation

    All derived classes have a few class parameters explained below.

    Attributes
    ----------

    EXPECTED_PARAMS, OPT_PARAMS : list of tuple, class attribute
        List of all expected parameters (resp. optional parameters) for this medium to be well defined.
        The format is `(param_name, param_type)` where `param_name` is a `str` and
        `param_type` a `type`.
    MEDIUM_TYPE : str, class attribute
        State the type of medium , used to determine the real physics hidden in the medium
        (to distinguish a fluid from and equivalent fluid for instance)
    MODEL : str, class attribute
        State which type of propagation model to use.
    name : str
        Medium's name (default to Generic Medium)
    omega : positive float or -1
        Last circular frequency for which the parameters have been updated
    """

    EXPECTED_PARAMS = []
    OPT_PARAMS = []
    MEDIUM_TYPE = 'generic'
    MODEL = ''

    def __init__(self, **params):
        self.omega = -1
        self.name = 'Generic Medium'

        if params:
            self.from_dict(params)

    def __str__(self):
        return '{} (type: {}, model: {})'.format(
            self.name,
            self.__class__.MEDIUM_TYPE,
            self.__class__.MODEL
        )

    def update_frequency(self, omega):
        """ Computes parameters' value for the given circular frequency

        Notes
        -----

        Implemented in derived classes
        """
        pass

    def _compute_missing(self):
        """ Computes the required constant parameters missing from the definition

        Notes
        -----

        Implemented in derived classes
        """
        pass

    def from_dict(self, parameters):
        """Reads medium definition from a hashmap of params.

        Load all possible & recognised parameters and run `_compute_missing` to complete
        the definition.

        Parameters
        ----------
        parameters : dict
            Linking parameter names and values.

        Raises
        ------
        LookupError
            If the parameter definition is incomplete (missing parameters listed in EXPECTED_PARAMS).
        """

        for param, param_type in self.__class__.EXPECTED_PARAMS:
            param_value = parameters.get(param)
            if param_value is None:
                raise LookupError('Unable to find definition of parameter "{}"'.format(param))
            else:
                setattr(self, param, param_type(param_value))
        for param, param_type in self.__class__.OPT_PARAMS:
            param_value = parameters.get(param)
            if param_value is not None:
                setattr(self, param, param_type(param_value))
        self.name = parameters.get('name', "Unnamed Medium")
        self._compute_missing()



#! /usr/bin/env python
# -*- coding:utf8 -*-
#
# eqf.py
#
# This file is part of pymls, a software distributed under the MIT license.
# For any question, please contact one of the authors cited below.
#
# Copyright (c) 2017
# 	Olivier Dazel <olivier.dazel@univ-lemans.fr>
# 	Mathieu Gaborit <gaborit@kth.se>
# 	Peter Göransson <pege@kth.se>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#

from numpy.lib.scimath import sqrt

from .medium import Medium
from .air import Air


class EqFluidJCA(Medium):
    """ Represents an equivalent fluid medium with the Johnson-Champoux-Allard approach

    The Johnson-Champoux-Allard model is described in references [1]_, [2]_ and [3]_.

    Attributes
    ----------
    phi : float
        Porosity
    sigma : float
        Flow resistivity
    alpha : float
        Tortuosity
    Lambda_prime : float
        Thermal characteristic length
    Lambda : float
        Viscous characteristic length

    Notes
    -----

    Populates the instance with the following attributes (and a few others left undocumented)

    `rho_eq_til`
        Frequency dependent equivalent density
    `alpha_til`
        Frequency dependent Johnson's tortuosity
    `alpha_prime_til`
        Frequency dependent Champoux-Allard's tortuosity
    `K_eq_til`
        Frequency dependent equivalent compressibility
    `c_eq_til`
        Frequency dependent equivalent speed of sound

    References
    ----------

    .. [1] D. L. Johnson *et al.*, "Theory of dynamic permeability and tortuosity in fluid-saturated porous media", Journal of fluid mechanics 176(1), 1987.
    .. [2] Y. Champoux & J.-F. Allard, "Dynamic tortuosity and bulk modulus in air-saturated porous media", Journale of Applied Physics, 1975, DOI: 10.1063/1.349482.
    .. [3] J.-F. Allard & N. Atalla, "Propagation of Sound in porous media: modelling sound absorbing materials", Wiley, 2009, ISBN: 978-0-470-74661-5.
    """

    MEDIUM_TYPE = 'eqf'
    MODEL = 'fluid'
    EXPECTED_PARAMS = [
        ('phi', float),  # Porosity
        ('sigma', float),  # Flow resistivity
        ('alpha', float),  # Tortuosity
        ('Lambda_prime', float),  # Thermal characteristic length
        ('Lambda', float),  # Viscous characteristic length
    ]

    OPT_PARAMS = [
        ('rho_1', float),  # Mass of solid per unit volume of aggregate
        ('nu', float),  # poisson ratio
        ('E', float),  # Young's modulus
        ('eta', float)  # viscosity
    ]

    def __init__(self, **params):
        self.phi = None
        self.sigma = None
        self.alpha = None
        self.Lambda_prime = None
        self.Lambda = None

        super().__init__(**params)

    @property
    def c(self):
        return self.c_eq_til

    @property
    def rho(self):
        return self.rho_eq_til

    def _compute_missing(self):
        """ Computes the required constant parameters missing from the definition

        For a JCA equivalent fluid, the shear modulus `N` is computed."""
        pass

    def update_frequency(self, omega):
        """ Computes the JCA parameters (see Notes on the class).

        Parameters
        ----------

        omega :
            Circular frequency of interest
        """

        #  Johnson et al model for rho_eq_til
        self.omega_0 = self.sigma*self.phi/(Air.rho*self.alpha)
        self.omega_infty = (self.sigma*self.phi*self.Lambda)**2/(4*Air.mu*Air.rho*self.alpha**2)
        self.F_JKD = sqrt(1+1j*omega/self.omega_infty)
        self.rho_eq_til = (Air.rho*self.alpha/self.phi)*(1+(self.omega_0/(1j*omega))*self.F_JKD)
        self.alpha_til = self.phi*self.rho_eq_til/Air.rho

        #  Champoux-Allard model for K_eq_til
        self.omega_prime_infty = (16*Air.nu_prime)/(self.Lambda_prime**2)
        self.F_prime_CA = sqrt(1+1j*omega/self.omega_prime_infty)
        self.alpha_prime_til = 1+self.omega_prime_infty*self.F_prime_CA/(2*1j*omega)
        self.K_eq_til = (Air.gamma*Air.P/self.phi)/(Air.gamma-(Air.gamma-1)/self.alpha_prime_til)

        self.c_eq_til = sqrt(self.K_eq_til/self.rho_eq_til)

        self.k = omega/self.c_eq_til



#! /usr/bin/env python
# -*- coding:utf8 -*-
#
# from_yaml.py
#
# Copyright © 2020 Mathieu Gaborit (matael) <mathieu@matael.org>
#
# Licensed under the "THE BEER-WARE LICENSE" (Revision 42):
# Mathieu (matael) Gaborit wrote this file. As long as you retain this notice
# you can do whatever you want with this stuff. If we meet some day, and you
# think this stuff is worth it, you can buy me a beer or coffee in return
#

import os
import yaml

from mediapack.eqf import EqFluidJCA
from mediapack.elastic import Elastic
from mediapack.pem import PEM
from mediapack.screen import Screen
from mediapack.fluid import Fluid


__MEDIUMCLASSES_MAP = {
    _.MEDIUM_TYPE: _ for _ in [EqFluidJCA, Elastic, PEM, Fluid, Screen]
}


def from_yaml(filename, force=None):
    """Reads medium definition from YAML file filename. Raises an IOError if the file
    is not found, ValueError if medium type is not known and a LookupError if the
    parameter definition is incomplete.

    One may set the optional argument 'force' to a medium type to force loading this one.
    """

    if not os.path.exists(filename):
        raise IOError('Unable to locate file {}'.format(filename))

    with open(filename) as fh:
        yaml_data = yaml.safe_load(fh)

    if yaml_data.get('medium_type') is None and force is None:
        raise LookupError('Unspecified medium type')

    if force is None:
        medium_class = __MEDIUMCLASSES_MAP.get(yaml_data['medium_type'])
    else:
        medium_class = force
    if medium_class is None:
        raise ValueError('Medium type is not known')
    else:
        medium = medium_class()
        medium.from_dict(yaml_data)
        return medium



#! /usr/bin/env python
# -*- coding:utf8 -*-
#
# screen.py
#
# This file is part of pymls, a software distributed under the MIT license.
# For any question, please contact one of the authors cited below.
#
# Copyright (c) 2017
# 	Olivier Dazel <olivier.dazel@univ-lemans.fr>
# 	Mathieu Gaborit <gaborit@kth.se>
# 	Peter Göransson <pege@kth.se>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#

from .pem import PEM
from .air import Air
from numpy.lib.scimath import sqrt
import numpy as np


class Screen(PEM):

    MEDIUM_TYPE = 'screen'

    def _compute_missing(self):
        self.lambda_ = self.E*self.nu/((1+self.nu)*(1-2*self.nu))

        self.rho_12 = 0  # Simplification on alpha ~= 1
        self.rho_11 = self.rho_1-self.rho_12
        self.rho_2 = self.phi*Air.rho
        self.rho_22 = self.rho_2-self.rho_12

    def update_frequency(self, omega):

        #  Simplified model for rho_eq_til & K_eq_til
        self.rho_eq_til = (Air.rho/self.phi)+self.sigma/(1j*omega)
        self.alpha_til = self.phi*self.rho_eq_til/Air.rho
        self.K_eq_til = Air.P/self.phi

        self.c_eq_til = sqrt(self.K_eq_til/self.rho_eq_til)

        self.rho_22_til = self.phi**2*self.rho_eq_til
        self.rho_12_til = self.rho_2-self.rho_22_til
        self.rho_11_til = self.rho_1-self.rho_12_til
        self.rho_til = self.rho_11_til-((self.rho_12_til**2)/self.rho_22_til)

        self.gamma_til = self.phi*(self.rho_12_til/self.rho_22_til-(1-self.phi)/self.phi)
        self.rho_s_til = self.rho_til+self.gamma_til**2*self.rho_eq_til

        if self.loss_type == 'anelastic':
            raise NotImplementedError("Anelastic losses not implemented yet")
            # *HAS* to be in update_frequency() if anelastic is considered
            # self.structural_loss=1+(b_hat*(1j*omega/beta_hat)**alpha_hat)/(1+(1j*omega/beta_hat)**alpha_hat)
        elif self.loss_type == 'structural':
            self.structural_loss = 1+1j*self.eta
        elif self.loss_type == 'none':
            self.structural_loss = 1
        else:
            raise ValueError('Unknown type of losses')

        self.N = self.E/(2*(1+self.nu))*self.structural_loss
        self.A_hat = (self.E*self.nu)/((1+self.nu)*(1-2*self.nu))*self.structural_loss
        self.P_hat = self.A_hat+2*self.N

        # Biot 1956 elastic coefficients
        self.R_til = self.K_eq_til*self.phi**2
        self.Q_til = ((1-self.phi)/self.phi)*self.R_til
        self.P_til = self.P_hat+self.Q_til**2/self.R_til

        delta_eq = omega*sqrt(self.rho_eq_til/self.K_eq_til)
        delta_s_1 = omega*sqrt(self.rho_til/self.P_hat)
        delta_s_2 = omega*sqrt(self.rho_s_til/self.P_hat)

        Psi = ((delta_s_2**2+delta_eq**2)**2-4*delta_eq**2*delta_s_1**2)
        sdelta_total = sqrt(Psi)

        delta_1 = sqrt(0.5*(delta_s_2**2+delta_eq**2+sdelta_total))
        delta_2 = sqrt(0.5*(delta_s_2**2+delta_eq**2-sdelta_total))
        delta_3 = omega*sqrt(self.rho_til/self.N)

        if np.abs(delta_1-delta_eq) < np.abs(delta_2-delta_eq):
            delta_1, delta_2 = delta_2, delta_1

        mu_1 = self.gamma_til*delta_eq**2/(delta_1**2-delta_eq**2)
        mu_2 = self.gamma_til*delta_eq**2/(delta_2**2-delta_eq**2)
        mu_3 = -self.gamma_til

        self.delta_s_1 = delta_s_1
        self.delta_s_2 = delta_s_2
        self.delta_1 = delta_1
        self.delta_2 = delta_2
        self.delta_3 = delta_3
        self.delta_eq = delta_eq
        self.sdelta_total = sdelta_total
        self.mu_1 = mu_1
        self.mu_2 = mu_2
        self.mu_3 = mu_3



#! /usr/bin/env python
# -*- coding:utf8 -*-
#
# air.py
#
# This file is part of pymls, a software distributed under the MIT license.
# For any question, please contact one of the authors cited below.
#
# Copyright (c) 2017
# 	Olivier Dazel <olivier.dazel@univ-lemans.fr>
# 	Mathieu Gaborit <gaborit@kth.se>
# 	Peter Göransson <pege@kth.se>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#

import numpy as np

from .medium import Medium


class Air(Medium):

    MEDIUM_TYPE = 'fluid'
    MODEL = MEDIUM_TYPE

    T = 293.15  # reference temperature [K]
    P = 1.01325e5  # atmospheric Pressure [Pa]
    gamma = 1.400  # polytropic coefficient []
    lambda_ = 0.0262  # thermal conductivity [W.m^-1.K^-1]
    mu = 0.1839e-4  # dynamic viscosity [kg.m^-1.s^-1]
    Pr = 0.710  # Prandtl's number []
    molar_mass = 0.29e-1  # molar mass [kg.mol^-1]
    rho = 1.213  # density [kg.m^-3]
    C_p = 1006  # (mass) specific heat capacity as constant pressure [J.K^-1]

    K = gamma*P  # adiabatic bulk modulus
    c = np.sqrt(K/rho)  # adiabatic sound speed
    Z = rho*c  # characteristic impedance
    C_v = C_p/gamma  # (mass) specific heat capacity as constant volume [J.K^-1]
    nu = mu/rho  # kinematic viscosity [m.s^-2]
    nu_prime = nu/Pr  # viscothermal losses

    def from_yaml(self, *args, **kwargs):
        pass

    def update_frequency(self, *args, **kwargs):
        pass



__VERSION__ = '0.4'

from .air import Air
from .fluid import Fluid
from .elastic import Elastic
from .eqf import EqFluidJCA
from .pem import PEM
from .screen import Screen

from .utils import from_yaml



#! /usr/bin/env python
# -*- coding:utf8 -*-
#
# fluid.py
#
# This file is part of pymls, a software distributed under the MIT license.
# For any question, please contact one of the authors cited below.
#
# Copyright (c) 2017
# 	Olivier Dazel <olivier.dazel@univ-lemans.fr>
# 	Mathieu Gaborit <gaborit@kth.se>
# 	Peter Göransson <pege@kth.se>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#

from .medium import Medium


class Fluid(Medium):
    """ Represent a fluid medium

    Attributes
    ----------

    rho : float
        density
    c : float
        sound speed
    """

    MEDIUM_TYPE = 'fluid'
    MODEL = MEDIUM_TYPE
    EXPECTED_PARAMS = [
        ('rho', float),  # Density
        ('c', float),  # Sound speed
    ]

    def __init__(self, **params):
        self.rho = None
        self.c = None
        self.k = None
        super().__init__(**params)
        self.Z = self.rho*self.c
        self.K = self.rho*self.c**2

    def update_frequency(self, omega):
        """ For a fluid, does nothing."""
        self.omega = omega
        self.k = omega/self.c
